#!/usr/bin/perl
#

package VCFFile;

use VCalendar;
use VEvent;
use VTodo;

sub new {
	my $class = shift;
	my $self = { @_ };

	bless $self, $class;
	return $self;
}

sub load_obj {
	my $self = shift;
	my $otype = shift;

	my $o = { };

	$o->{type} = $otype;
	$o->{attrs} = { };
	$o->{contents} = [ ];

	my $line;

	# Read keys and values

	while (defined ($line = <F>)) {
		$line =~ s/\r//;
		chomp($line);
		if ($line =~ /^([^:]+):(.+)/) {
			my($key, $value) = ($1, $2);

			if ($key eq 'BEGIN') {
				my $newo = $self->load_obj($value);
				push(@{$o->{contents}}, $newo);
				next;
			}

			if ($key eq 'END') {
				if ($value ne $otype) {
					die "File error: expected END:$otype got END:$value";
				}

				my $obj;

				if ($otype eq 'VEVENT') {
					$obj = new VEvent();
				} elsif ($otype eq 'VTODO') {
					$obj = new VTodo();
				} elsif ($otype eq 'VCALENDAR') {
					$obj = new VCalendar();
				} else {
					die "Unknown type: $otype";
				}

				$obj->{attrs} = $o->{attrs};
				$obj->{contents} = $o->{contents};

				return $obj;
			}

			$o->{attrs}->{$key} = $value;
		} else {
			# Ignore this line
		}
	}

	die "Unable to load: ran off the end of file";
}

sub load {
	my $self = shift;
	my $path = shift;

	if (!open(F, "<$path")) {
		die "Unable to load $path: $!";
	}

	# Read BEGIN:VCALENDAR line

	my $line = <F>;
	$line =~ s/\r//;
	chomp($line);

	if ($line ne 'BEGIN:VCALENDAR') {
		die "Unable to load $path: file begins with ($line), expecting (BEGIN:VCALENDAR)";
	}

	my $o = $self->load_obj('VCALENDAR');

	# Move the vcalendar contents up in the object data structures

	$self->{type} = 'VCALENDAR';
	$self->{calendar} = $o;
	$self->{attrs} = $o->{attrs};
	$self->{contents} = $o->{contents};

}

1;

